import os
import zipfile
from typing import Optional

import requests
import typer

import re
import sys
import base64
from urllib import parse
from pprint import pprint


def create_zip_file(file_path, target_dir):
    directories_to_ignore = [".git", ".github", "dist"]
    zipobj = zipfile.ZipFile(file_path, "w", zipfile.ZIP_DEFLATED)
    rootlen = len(target_dir) + 1

    print("create_zip_file")
    pprint(file_path)
    pprint(target_dir)

    for base, dirs, files in os.walk(target_dir):
        if any(ignore_directory in base for ignore_directory in directories_to_ignore):
            print("skip:" + base)
            continue

        for file in files:
            pprint(file)
            fn = os.path.join(base, file)
            zipobj.write(fn, fn[rootlen:])


def upload(
    username: Optional[str], password: Optional[str], project_id: str, zipfile: str
):
    authority = "www.pling.com"
    baseUrl = "https://" + authority + "/"
    the_file = base64.b64encode(open(zipfile, "rb").read())

    headers = {
        "authority": authority,
        "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
        "accept-language": "en-GB,en-US;q=0.9,en;q=0.8",
        "cache-control": "no-cache",
        "dnt": "1",
        "pragma": "no-cache",
        "referer": baseUrl,
        "sec-ch-ua": '"Not_A Brand";v="99", "Google Chrome";v="109", "Chromium";v="109"',
        "sec-ch-ua-mobile": "?0",
        "sec-ch-ua-platform": '"Linux"',
        "sec-fetch-dest": "document",
        "sec-fetch-mode": "navigate",
        "sec-fetch-site": "same-origin",
        "sec-fetch-user": "?1",
        "upgrade-insecure-requests": "1",
        "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36",
    }

    login_data = {
        "csrf": "",
        "email": username,
        "password": password,
        "next": "/",
        "remember_me": "1",
    }

    client = requests.Session()
    client.get(baseUrl, headers=headers)
    login_page = client.get(baseUrl + "login/", headers=headers)
    result = re.search(
        r"name=\"csrf\" value=\"(.*?)\"", login_page.content.decode("UTF-8")
    )
    login_data["csrf"] = result.group(1)

    headers["referer"] = baseUrl + "login/"
    login_response = client.post(baseUrl + "login/", login_data, headers=headers)

    if b"Incorrect login and/or password" in login_response.content:
        typer.echo("failed login")
        sys.exit(1)

    edit_page = client.get(baseUrl + "p/" + project_id + "/edit", headers=headers)
    result = re.search(r"var fileUri = '(.*?)'", edit_page.content.decode("UTF-8"))
    upload_url = result.group(1)
    result = re.search(r"\"owner_id\", '(.*?)'", edit_page.content.decode("UTF-8"))
    owner_id = result.group(1)
    result = re.search(r"client_id = '(.*?)'", edit_page.content.decode("UTF-8"))
    client_id = result.group(1)
    result = re.search(
        r"data-ppload-collection-id=\"(.*?)\"", edit_page.content.decode("UTF-8")
    )
    collection_id = result.group(1)

    headers["acccept"] = "application/json, text/javascript, */*; q=0.01"
    headers["origin"] = baseUrl
    headers["sec-fetch-dest"] = "empty"
    headers["sec-fetch-mode"] = "cors"
    headers["sec-fetch-site"] = "same-site"
    del headers["sec-fetch-user"]
    del headers["upgrade-insecure-requests"]

    params = dict(parse.parse_qsl(parse.urlsplit(upload_url).query))

    post_data = {
        "collection_id": collection_id,
        "id": "1676575635",
        "owner_id": owner_id,
        "format": "json",
        "client_id": client_id,
        "name": "file",
        "filename": "archive.tgz",
    }

    files = {"file": (zipfile, the_file, "application/zip")}
    response = client.post(
        upload_url, params=params, headers=headers, data=post_data, files=files
    )

    if '"status":"success"' in response.content.decode("UTF-8"):
        return True
    else:
        typer.echo("Failed publishing: " + zipfile)
        typer.echo("\n")
        typer.echo(response.content.decode("UTF-8"))
        return False



import os
from shutil import rmtree
from typing import Optional
import calendar
import time

import typer
from pling_publisher.utils import (
    create_zip_file,
    upload,
)

app = typer.Typer()


@app.command()
def publisharchive(
    file: str = os.getcwd(),
    username: Optional[str] = os.environ.get("PLING_USERNAME", None),
    password: Optional[str] = os.environ.get("PLING_PASSWORD", None),
    project_id: Optional[str] = os.environ.get("PLING_PROJECT_ID", None),
):
    file = os.path.abspath(file)

    if upload(username, password, project_id, file):
        typer.echo("Uploaded.")


@app.command()
def publish(
    directory: str = os.getcwd(),
    project_id: Optional[str] = os.environ.get("PLING_PROJECT_ID", None),
    username: Optional[str] = os.environ.get("PLING_USERNAME", None),
    password: Optional[str] = os.environ.get("PLING_PASSWORD", None),
):
    full_zip_path = build(
        directory=directory,
    )

    if upload(username, password, project_id, full_zip_path):
        typer.echo("Uploaded.")


@app.command()
def build(directory: str = os.getcwd()):
    directory = os.path.abspath(directory)

    current_GMT = time.gmtime()

    upload_filename = "archive-" + str(calendar.timegm(current_GMT)) + ".zip"

    dist_directory = os.path.join(directory, "dist")

    if os.path.isdir(dist_directory):
        rmtree(dist_directory)

    os.mkdir(dist_directory)

    full_zip_path = os.path.join(dist_directory, upload_filename)
    create_zip_file(full_zip_path, directory)
    typer.echo(f"Created zip file: {full_zip_path}")
    return full_zip_path



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pling_publisher']

package_data = \
{'': ['*']}

install_requires = \
['requests', 'typer']

entry_points = \
{'console_scripts': ['pling = pling_publisher.cli:app']}

setup_kwargs = {
    'name': 'pling-publisher',
    'version': '0.1.7',
    'description': '',
    'long_description': '# Pling Publisher\nTool to upload to [gnome-look.org](https://gnome-look.org), [store.kde.org](https://store.kde.org), [www.pling.com](https://https://www.pling.com/).\n\n![Build Status](https://github.com/dmzoneill/pling-publisher/actions/workflows/main.yml/badge.svg)\n\n\nYou can find this python module on [https://pypi.org/project/pling-publisher/](https://pypi.org/project/pling-publisher/)\n\n\n## Install\n```console\npip install pling-publisher\n```\n\n## How to use\n```console\npling build # build an archive from the current folder.\npling publish --username <USERNAME> --password <PASSWORD> --project-id <PROJECT_ID> --file <PATH>\npling --help # for help :)\n```\n\nYou can also provide your username, password and project_id via environment variables (PLING_USERNAME, PLING_PASSWORD, PLING_PROJECT_ID).\n\n\n## Support\nFeel free to submit a pull request\n',
    'author': 'David O Neill',
    'author_email': 'dmz.oneill@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/dmzoneill/pling-publisher',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
